""" This example show use case off complex decisions
"""

from src.pysidetap.processor import DTProcessor

min_temp = 48.0
max_temp = 55.0
cooldown = 30*60
warmup = 60*60
DTableEQ = [
    {
        'fields': {
            'reservoar_temp': {'op':'<', 'value':min_temp},
            'cooldown': {'op':'>', 'value':cooldown},
            'heat_status': {'op':'==', 'value':'off'}
        },
        'return': {'heat':'on', 'message':'reservoar heat_status was off and temp was under min_temp and cooldown was passed '}
    },
    {
        'fields': {
            'reservoar_temp': {'op':'<=', 'value':max_temp},
            'cooldown': {'op':'<', 'value':warmup},
            'heat_status': {'op':'==', 'value':'on'}
        },
        'return': {'heat':'on', 'message':'reservoar heat_status was on and temp was under max_temp and warmup is not passed '}
    },
    {
        'fields': {
            'reservoar_temp': {'op':'>=', 'value':max_temp},
            'cooldown': {'op':'<', 'value':warmup},
            'heat_status': {'op':'==', 'value':'on'}
        },
        'return': {'heat':'on', 'message':'reservoar heat_status was on and temp was oves max_temp and warmup is not passed '}
    },
    {
        'return': {'heat':'off', 'message':'something else then heat off'}
    },
]

p = DTProcessor(DTableEQ)
for heat_status in ['on', 'off']:
    for temp in range(40, 60):
        for cooldown in range(0,2*3600, 60):
            r = p.process({'reservoar_temp':temp, 'cooldown':cooldown, 'heat_status':heat_status})
            if r == None:
                pass
            print(temp, cooldown, heat_status, r)



"""Module processor.py implements the Decision table processor."""
from typing import List

try:
    import pysidetap.libs.operations as operations
except ModuleNotFoundError:
    import libs.operations as operations


class DTProcessor:
    """DTProcessor Decision table processor class."""

    _d_table: List[dict]
    _op: dict = {
        '==': operations.op_eq,
        '!=': operations.op_noteq,
        '>': operations.op_gt,
        '>=': operations.op_gteq,
        '<': operations.op_lt,
        '<=': operations.op_lteq,
    }

    def __init__(self, table: List[dict] = None) -> None:
        if table:
            self.load_table(table)

    def load_table(self, table: List[dict]) -> None:
        """load_table Load decision table into processor.

        Args
        ----
            table: List[dict], optional
                table of fields (operation and values) and return.

        Example
        -------
            Decision Table:
            [
                {
                    'fields': {
                        'field1': {'op':'==','value':1},
                    },
                    'return': 'field1==1'
                },
                {
                    'fields': {
                        'field1': {'op':'!=','value':1},
                    },
                    'return': 'field1!=1'
                }
            ]
        """
        self._d_table = table

    def process(self, values: dict) -> any:
        """Process decision table over values.

        This function find and return field 'return' from Decision Table,
        when all operands in row by 'fields' are True.

        Args
        ----
            values : dict
                Dict of fields that will be evaluated

        Example
        -------
            {'field1': 2}

        Returns
        -------
            any
                return value from decision table evaluated row
        """
        for table_row in self._d_table:
            all_pass = True
            if table_row.get('fields'):
                # if decision table row have fields let's evaluate them
                for table_field_key, table_field in table_row['fields'].items():
                    # for each field evaluate operation.
                    # on left side is value on right side is table value
                    res = self._op[table_field['op']](
                        values[table_field_key], table_field['value']
                    )
                    if not res:
                        all_pass &= False
            if all_pass:
                return table_row['return']
        return None



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""Setup dot py."""
from __future__ import absolute_import, print_function

# import re
from glob import glob
from os.path import basename, dirname, join, splitext

from setuptools import find_packages, setup


def read(*names, **kwargs):
    """Read description files."""
    path = join(dirname(__file__), *names)
    with open(path, encoding=kwargs.get('encoding', 'utf8')) as fh:
        return fh.read()


# previous approach used to ignored badges in PyPI long description
# long_description = '{}\n{}'.format(
#     re.compile(
#         '^.. start-badges.*^.. end-badges',
#         re.M | re.S,
#         ).sub(
#             '',
#             read('README.rst'),
#             ),
#     re.sub(':[a-z]+:`~?(.*?)`', r'``\1``', read(join('docs', 'CHANGELOG.rst')))
#     )

long_description = '{}\n{}'.format(
    read('README.rst'),
    read('CHANGELOG.rst'),
)

setup(
    name='pysidetap',
    version='0.0.8',
    description='A skeleton template for Python projects.',
    long_description=long_description,
    long_description_content_type='text/x-rst',
    license='MIT License',
    author='Martin Kapinos',
    author_email='matkapi19@gmail.com',
    url='https://github.com/matkapi/pysidetap',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    py_modules=[splitext(basename(i))[0] for i in glob("src/*.py")],
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        # complete classifier list:
        # http://pypi.python.org/pypi?%3Aaction=list_classifiers
        'Development Status :: 4 - Beta',
        # 'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        'License :: OSI Approved :: MIT License',
        'Intended Audience :: Science/Research',
        'Natural Language :: English',
        'Operating System :: POSIX',
        'Operating System :: MacOS',
        'Operating System :: Microsoft',
        'Programming Language :: Python :: 3.10',
        'Topic :: Scientific/Engineering :: Bio-Informatics',
    ],
    project_urls={
        'webpage': 'https://github.com/matkapi/pysidetap',
        'Documentation': 'https://pysidetap.readthedocs.io/en/latest/',
        'Changelog': 'https://github.com/matkapi/pysidetap/blob/master/CHANGELOG.rst',
        'Issue Tracker': 'https://github.com/matkapi/pysidetap/issues',
        'Discussion Forum': 'https://github.com/matkapi/pysidetap/discussions',
    },
    keywords=[
        'decision-table',
    ],
    python_requires='>=3.7, <4',
    install_requires=[
        # https://stackoverflow.com/questions/14399534
        # 'matplotlib>=3',
    ],
    extras_require={
        # eg:
        #   'rst': ['docutils>=0.11'],
        #   ':python_version=="2.6"': ['argparse'],
    },
    setup_requires=[
        #   'pytest-runner',
        #   'setuptools_scm>=3.3.1',
    ],
    entry_points={
        'console_scripts': [
            'samplecli1= pysidetap.cli_int1:main',
        ]
        #
    },
    # cmdclass={'build_ext': optional_build_ext},
    # ext_modules=[
    #    Extension(
    #        splitext(relpath(path, 'src').replace(os.sep, '.'))[0],
    #        sources=[path],
    #        include_dirs=[dirname(path)]
    #    )
    #    for root, _, _ in os.walk('src')
    #    for path in glob(join(root, '*.c'))
    # ],
)
